
C 		p a t c h _ c o n t o u r

C Derived from fdmap_xplor, March 2006, Jim Warwicker, UoM

C Make mol, potl and efld formatted xplor maps (from fd o/p), with o/p
C non-zero only in solvent regions (incl nn points for efld).
C Contour (2D on shell and optionally 3D) potential .
C Maybe later on we will add optional contouring for E-field too.
C Output text files of contoured patches/volumes.

C Also translate the map-generating coords to the xplor map origin.

C Current i/p:

C $MOLECULE.pdb
C fd_$MOLECULE.pot	FD potl map from fdmap.pdb coords
C fd_$MOLECULE.mol	FD mol map from fdmap.pdb coords
C fd_$MOLECULE.itr	FD itr map which tells us where we iterated (so
C			that we can exclude non-iterating points)

C Current o/p (txt extensions indicate ASCII X-PLOR format maps):

C $MOLECULE_map.pdb	coords translated for xplor map origin
C $MOLECULE_potl.txt	potential map (solvent region only) in mV
C $MOLECULE_mol.txt	molecule map - normally 4 or 78
C $MOLECULE_efld.txt	electric field (solvent region only) in mV/A
C $MOLECULE_peaks.inf	info about peaks

C $MOLECULE_patch.pdb	Bfac field has contour code, for mol graphics
C $MOLECULE_shell.pdb	Shell pts, coloured as for X_patch.pdb

C Updated May 2007 to also make:
C $MOLECULE_potl.hist	multiple added fields to histogram max potl
C And again Oct 2007 to add absolute potl max field to this file

	REAL		potl (150,150,150), mole (150,150,150),
     1			zero (3), step, cell (3), zero_std (3),
     2			sect1 (10000), sect2 (10000),efld(150,150,150),
     3			dfld (150,150,150), enrgy (150,150,150),
     4			sect4 (10000), sect5 (10000)
	INTEGER		dims (3), dims_std (3)
	LOGICAL		hit (150,150,150), solv (150,150,150)
	LOGICAL*1	itr (150,150,150)
	CHARACTER*80	header, rec
	CHARACTER*60	molecule, protein_potl, patch_contour,
     1			np_contour, rwb
	CHARACTER*4	sign_here
	CHARACTER*8	remark
	CHARACTER*6	atom
	CHARACTER*4	atyp, res
	CHARACTER*3	zyx
	CHARACTER*1	chain, chn_m (5)
	LOGICAL		mol_here, add_atom, get_shell,
     1			more_maps, prot_potl, white_out
	REAL		peak_sums (20,5000)
	INTEGER		npeaks (20), peak_pts (20,5000)
	CHARACTER*6	atom_str (50000)
	CHARACTER*4	atyp_str (50000), res_str (50000)
	CHARACTER*1	chain_str (50000)
	REAL		xyz_str (50000,3), np_cut
	INTEGER		nres_str (50000), cb_str (50000)

	COMMON	/maps/		potl, efld, mole, itr, dims, step, solv
	COMMON	/peaks/		npeaks, peak_sums, peak_pts
	COMMON	/ats_str/	nstr,atom_str,atyp_str,res_str,
     1				chain_str,nres_str,xyz_str,cb_str
	COMMON	/patches/	potl_cut, np_cut
	COMMON	/whitener/	white_out
	COMMON	/moley/		molecule, iml

	DATA chn_m		/ '1', '2', '3', '4', '5'/

2010	FORMAT (/,I8,/,A8,/,A8,/,A8)
2100	FORMAT (9I8)
2200	FORMAT (6E12.5)
2210	FORMAT (A3)
2300	FORMAT (I8)
2400	FORMAT (I8)
2500	FORMAT (2(E12.4,1X))
3000	FORMAT (A80)
3100	FORMAT (A6, 7X, A4, A4, A1, I4, 4X, 3F8.3)
3300	FORMAT (A1,2X,I4,2X,A4)
3800	FORMAT (' patches for molecule +/-contour npcontour = ',
     1          A20,2(1X,F7.1),' mV')
3820	FORMAT (' grid step = ',F8.3,' A, area step = ',F8.3,' Asq')
4220	FORMAT (2X,F6.2,3X,I5,3X,F8.2,3X,I6)
4400	FORMAT (/,'peak +- solvent potls mV = ',2F7.1,
     1' at dp/dn  = ',2F6.1,/)
4420	FORMAT (/,'peak +- solvent potls mV = ',2F7.1,/)
5200	FORMAT ('Ecut  >05  >10  >15  >20  >25  >30  >35  >40  >45
     1  >50  >55  >60  >65  >70 dmean',/)
5400	FORMAT (F4.1,14(2X,I3),F6.1)
5640	FORMAT ('grid step = ',F6.2,/)

C Set o/p of D field and D.E maps to investigate inside protein.

	more_maps	= .FALSE.

C Set up the run from environment variables:

C MOLECULE	e.g. handle files as molecule'.pot' etc
C PROTEIN_POTL	if set to 'yes', then include potl in protein/zero otherwise
C PATCH_CONTOUR	+ve value at which to contour patches (e.g. 50 mV)
C NP_CONTOUR	+ve value, then between -this and +this we contour np
C		could be the same as PATCH_CONTOUR or lower (not higher)

	CALL	GETENV ('RED_WHITE_BLUE', rwb)
	CALL	leader (rwb, irb)
	IF ((rwb(1:3).EQ.'YES').OR.(rwb(1:3).EQ.'yes')) THEN
	  white_out	= .TRUE.
	ELSE
	  white_out	= .FALSE.
	END IF

	CALL	GETENV ('MOLECULE', molecule)
	CALL	leader (molecule, iml)

	CALL	GETENV ('PROTEIN_POTL', protein_potl)
	CALL	leader (protein_potl, ipp)
	prot_potl	= .FALSE.
	IF (protein_potl (1:ipp) .EQ. 'yes') THEN
	  prot_potl	= .TRUE.
	END IF

	CALL	GETENV ('PATCH_CONTOUR', patch_contour)
	CALL	leader (patch_contour, ipc)
	IF (ipc .EQ. 0) STOP 'PATCH_CONTOUR not set (+ve mV)'
	READ (patch_contour (1:ipc),*) potl_cut
	IF (potl_cut .LT. 0.) STOP 'potl_cut is negative'

	CALL	GETENV ('NP_CONTOUR', np_contour)
	CALL	leader (np_contour, inc)
	IF (inc .EQ. 0) STOP 'NP_CONTOUR not set (+ve mV)'
	READ (np_contour (1:inc),*) np_cut
	IF (np_cut .LT. 0.) STOP 'np_cut is negative'
	IF (np_cut .GT. potl_cut) STOP 'overlapping -/0/+ patches'

C Read in the fdcalc potential file.

	OPEN (UNIT=1,NAME='fd_'//molecule(1:iml)//'.pot',
     1	      TYPE='OLD',FORM='UNFORMATTED')
	READ (1) zero_std, dims_std, step_std
	DO 200 k	= 1, dims_std (3)
	  DO 200 j	= 1, dims_std (2)
	    READ (1) (potl (i,j,k), i=1,dims_std (1))
200	CONTINUE						! End j,k
	CLOSE (UNIT=1)

C Work out the X-PLOR map parameters, as I understand them.

	cell (1)	= step_std*FLOAT (dims_std(1)-1)
	cell (2)	= step_std*FLOAT (dims_std(2)-1)
	cell (3)	= step_std*FLOAT (dims_std(3)-1)
	ntitle		= 3
	remark		= ' REMARKS'
	zyx		= 'ZYX'
	nxy		= dims_std (2) * dims_std (1)
	iflag		= -9999
	av		= 0.
	sd		= 1.

C Read in the fdcalc file that records where we had non-zero potential.

	OPEN (UNIT=1,NAME='fd_'//molecule(1:iml)//'.itr',
     1	      TYPE='OLD',FORM='UNFORMATTED')
	READ (1) zero, dims, step
	CALL 	 grid_check (zero,dims,step,zero_std,dims_std,step_std)
	DO 215 k	= 1, dims (3)
	  DO 215 j	= 1, dims (2)
	    READ (1) (itr (i,j,k), i=1,dims (1))
215	CONTINUE						! End j,k
	CLOSE (UNIT=1)

C Read in the molecule dielectric file.

	OPEN (UNIT=1,NAME='fd_'//molecule(1:iml)//'.mol',
     1	      TYPE='OLD',FORM='UNFORMATTED')
	READ (1) zero, dims, step
	CALL 	 grid_check (zero,dims,step,zero_std,dims_std,step_std)
	DO 300 k	= 1, dims (3)
	  DO 300 j	= 1, dims (2)
	    READ (1) (mole (i,j,k), i=1,dims(1))
300	CONTINUE						! End j,k
	CLOSE (UNIT=1)

C Make an array to flag whether molecule or solvent, assuming that
C any diel less than 78.4 will be molecule (e.g use 78.3 for '80/80' run)

	pk_mole		= 0.
	DO 310 k	= 1, dims (3)
	  DO 310 j	= 1, dims (2)
	    DO 310 i	= 1, dims (1)
	      IF (mole (i,j,k) .GT. pk_mole) pk_mole = mole (i,j,k)
	      IF (mole (i,j,k) .LE. 78.35) THEN
		solv (i,j,k) = .FALSE.
	      ELSE
		solv (i,j,k) = .TRUE.
	      END IF
310	CONTINUE						! End i,j,k
	IF (pk_mole .LE. 78.35) STOP 'fdmap_xplor: pk_mole<78.35'

C Move the coordinates into the map.
C Write out moved coords - and store so that we can pick out potl peak patch.
C Store as unmoved coords, so we can write out a colour-coded set in the
C same space as original coords.

C cb_str (int) will flag (generally 0) whether atom is cb (1) and AS (2).

	nstr			= 0
C?? Actually just use the 'cleaned' - see patches.pl script.
	OPEN (UNIT=3, NAME='tmp_es.pdb', TYPE='OLD')
C??	OPEN (UNIT=3, NAME=molecule(1:iml)//'.pdb', TYPE='OLD')
	OPEN (UNIT=4,NAME=molecule(1:iml)//'_map.pdb',TYPE='NEW')
C??	READ (3,3000) header
C??	WRITE (4,3000) header

160	READ (3,3000,END=180) rec
	IF ((rec(1:4).EQ.'ATOM').OR.(rec(1:4).EQ.'HETA')) THEN
	  READ (rec,3100) atom, atyp, res, chain, nres, x,y,z
	  IF (chain .EQ. ' ') chain = 'A'
	  xm		= x - zero (1)
	  ym		= y - zero (2)
	  zm		= z - zero (3)
	  WRITE (4,3100) atom, atyp, res, chain, nres, xm,ym,zm
	  nstr			= nstr + 1
	  cb_str (nstr)		= 0
	  IF (nstr .GT. 50000) STOP 'nstr > 50000'
	  atom_str (nstr)	= atom
	  atyp_str (nstr)	= atyp
	  res_str (nstr)	= res
	  chain_str (nstr)	= chain
	  nres_str (nstr)	= nres
	  xyz_str (nstr,1)	= x
	  xyz_str (nstr,2)	= y
	  xyz_str (nstr,3)	= z
	END IF
	GOTO 160

180	CLOSE (UNIT=3)
	CLOSE (UNIT=4)
C Coordinate file colour-coded by patches, written in Ppeaks_find.
	OPEN (UNIT=4,NAME=molecule(1:iml)//'_patch.pdb',TYPE='NEW')
C Coordinate files with extended Bfac field for pos and abs potl maxima.
	OPEN (UNIT=7,NAME=molecule(1:iml)//'_potl.hist',TYPE='NEW')

C Write the transcribed potential and molecule grids (zero potl in protein).

	OPEN (UNIT=1,NAME=molecule(1:iml)//'_potl.txt',
     1	      TYPE='NEW',FORM='FORMATTED')
	OPEN (UNIT=2,NAME=molecule(1:iml)//'_mole.txt',
     1	      TYPE='NEW',FORM='FORMATTED')
	WRITE (1,2010) ntitle, remark, remark, remark
	WRITE (1,2100) dims(1),1,dims(1),
     1		       dims(2),1,dims(2),
     2		       dims(3),1,dims(3)
	WRITE (1,2200) (cell(m),m=1,3), 90.,90.,90.
	WRITE (1,2210) zyx
	WRITE (2,2010) ntitle, remark, remark, remark
	WRITE (2,2100) dims(1),1,dims(1),
     1		       dims(2),1,dims(2),
     2		       dims(3),1,dims(3)
	WRITE (2,2200) (cell(m),m=1,3), 90.,90.,90.
	WRITE (2,2210) zyx

	DO 420 k	= 1, dims (3)
	  WRITE (1,2300) k
	  WRITE (2,2300) k
	  ij		= 0
	  DO  400 j	= 1, dims (2)
	    DO 400 i	= 1, dims (1)
	      ij	= ij + 1
	      sect1 (ij) = potl (i,j,k)
	      IF ((.NOT.solv(i,j,k))) THEN
		IF (.NOT. prot_potl) sect1 (ij) = 0.
	      END IF
	      IF (.NOT. itr (i,j,k)) sect1 (ij) = 0.
	      sect2 (ij) = mole (i,j,k)
400	  CONTINUE
	  WRITE (1,2200) (sect1(m),m=1,ij)
	  WRITE (2,2200) (sect2(m),m=1,ij)
420	CONTINUE
	WRITE (1,2400) iflag
	WRITE (1,2500) av, sd
	WRITE (2,2400) iflag
	WRITE (2,2500) av, sd
	CLOSE (UNIT=1)
	CLOSE (UNIT=2)

C Work out and write an E field magnitude map.
C Note that we only have non-zero grid points where this point plus the 6 nn
C on xyz are solvent points - this is going to be quite restrictive.
C The E-field estimate is very simple - could be improved.

C Optionally (more_maps/investigative) write out D field and D.E maps.

	OPEN (UNIT=1,NAME=molecule(1:iml)//'_efld.txt',
     1	      TYPE='NEW',FORM='FORMATTED')
	WRITE (1,2010) ntitle, remark, remark, remark
	WRITE (1,2100) dims(1),1,dims(1),
     1		       dims(2),1,dims(2),
     2		       dims(3),1,dims(3)
	WRITE (1,2200) (cell(m),m=1,3), 90.,90.,90.
	WRITE (1,2210) zyx

	IF (more_maps) THEN
	  OPEN (UNIT=2,NAME=molecule(1:iml)//'_dfld.txt',
     1	        TYPE='NEW',FORM='FORMATTED')
	  WRITE (2,2010) ntitle, remark, remark, remark
	  WRITE (2,2100) dims(1),1,dims(1),
     1		         dims(2),1,dims(2),
     2		         dims(3),1,dims(3)
	  WRITE (2,2200) (cell(m),m=1,3), 90.,90.,90.
	  WRITE (2,2210) zyx
	  OPEN (UNIT=3,NAME=molecule(1:iml)//'_enrgy.txt',
     1	        TYPE='NEW',FORM='FORMATTED')
	  WRITE (3,2010) ntitle, remark, remark, remark
	  WRITE (3,2100) dims(1),1,dims(1),
     1		         dims(2),1,dims(2),
     2		         dims(3),1,dims(3)
	  WRITE (3,2200) (cell(m),m=1,3), 90.,90.,90.
	  WRITE (3,2210) zyx
	END IF

	DO 440 k		= 1, dims (3)
	  DO  440 j		= 1, dims (2)
	    DO 440 i		= 1, dims (1)
	      IF ((k.EQ.1) .OR. (k.EQ.dims(3))) THEN
	        efld (i,j,k)	= 0.
	        GOTO 440
	      END IF
	      IF ((j.EQ.1) .OR. (j.EQ.dims(2))) THEN
	        efld (i,j,k)	= 0.
	        GOTO 440
	      END IF
	      IF ((i.EQ.1) .OR. (i.EQ.dims(1))) THEN
	        efld (i,j,k)	= 0.
	        GOTO 440
	      END IF
	      IF (.NOT. itr (i,j,k)) THEN
		efld (i,j,k)	= 0.
		GOTO 440
	      END IF
	      mol_here		= .FALSE.
	      IF (.NOT.solv(i,j,k)) mol_here = .TRUE.
	      IF (.NOT.solv(i-1,j,k)) mol_here = .TRUE.
	      IF (.NOT.solv(i+1,j,k)) mol_here = .TRUE.
	      IF (.NOT.solv(i,j-1,k)) mol_here = .TRUE.
	      IF (.NOT.solv(i,j+1,k)) mol_here = .TRUE.
	      IF (.NOT.solv(i,j,k-1)) mol_here = .TRUE.
	      IF (.NOT.solv(i,j,k+1)) mol_here = .TRUE.
	      IF (mol_here) THEN
		efld (i,j,k)	= 0.
		GOTO 440
	      END IF
	      Ex	= potl (i+1,j,k) - potl (i-1,j,k)
	      Ey	= potl (i,j+1,k) - potl (i,j-1,k)
	      Ez	= potl (i,j,k+1) - potl (i,j,k-1)
	      efld_here		= SQRT (Ex*Ex + Ey*Ey + Ez*Ez)
	      efld (i,j,k)	= efld_here / (2.*step)
440	CONTINUE

	DO 480 k	= 1, dims (3)
	  WRITE (1,2300) k
	  IF (more_maps) THEN
	    WRITE (2,2300) k
	    WRITE (3,2300) k
	  END IF
	  ij		= 0
	  DO  460 j	= 1, dims (2)
	    DO 460 i	= 1, dims (1)
	      ij	= ij + 1
	      sect1 (ij) = efld (i,j,k)
	      IF (more_maps) THEN
		dfld (i,j,k)	= mole(i,j,k) * efld(i,j,k)
		sect4 (ij)	= dfld(i,j,k)
		enrgy (i,j,k)	= dfld(i,j,k) * efld(i,j,k)
		sect5 (ij)	= enrgy(i,j,k)
	      END IF
460	  CONTINUE
	  WRITE (1,2200) (sect1(m),m=1,ij)
	  IF (more_maps) THEN
	    WRITE (2,2200) (sect4(m),m=1,ij)
	    WRITE (3,2200) (sect5(m),m=1,ij)
	  END IF
480	CONTINUE
	WRITE (1,2400) iflag
	WRITE (1,2500) av, sd
	CLOSE (UNIT=1)
	IF (more_maps) THEN
	  WRITE (2,2400) iflag
	  WRITE (2,2500) av, sd
	  WRITE (3,2400) iflag
	  WRITE (3,2500) av, sd
	  CLOSE (UNIT=2)
	  CLOSE (UNIT=3)
	END IF

C For now, look at just potential contours - we can add E-field back in
C later (code below if we choose).

	OPEN (UNIT=1, NAME=molecule(1:iml)//'_peaks.inf', TYPE='NEW')
	WRITE (1,3800) molecule (1:iml), potl_cut, np_cut
	WRITE (1,3820) step_std, step_std*step_std
	WRITE (1,*)

	CALL	Ppeaks_find (zero_std,step_std)

C??wednesday
	CLOSE (UNIT=1)

C The basic parameter is the E-field at which we cut-off peaks - try a
C range of Ecut values and track the number of peaks we get.
C At each Ecut we look for any unprocessed efld pts above Ecut, and move
C out until all nn pts above Ecut are included in this peak.
C Also look for +/- potential peaks - easier.

C	CALL	Ppeaks_find (zero_std,step_std)

C	DO 600 n		= 1, 20
C	  Ecut			= Ecut_base + Ecut_inc*FLOAT(n-1)
C	  ic			= n
C	  CALL	Epeaks_find (Ecut, ic, bigE, nbigE,
C     1			     dbigE, zero_std,step_std, dbigET)
C	  WRITE (1,4220) Ecut,npeaks(n),bigE,nbigE
C600	CONTINUE

C	CLOSE (UNIT=1)

	CALL	EXIT
	END


	SUBROUTINE	grid_check (zero,dims,step,zero_std,dims_std,
C	**********	**********
     1				    step_std)

	REAL		zero (3), zero_std (3)
	INTEGER		dims (3), dims_std (3)

	IF (ABS(step-step_std) .GT. 0.0001) STOP 'step mismatch'
	DO 200 n	= 1, 3
	  IF (ABS(zero(n)-zero_std(n)) .GT. 0.0001) STOP 'zero mismtch'
	  IF (dims(n).NE.dims_std(n)) STOP 'dims mismatch'
200	CONTINUE

	RETURN
	END


	SUBROUTINE	leader (char_array, itop)
C	**********	******

	CHARACTER*60	char_array

	itop	= 1
	DO WHILE (char_array (itop:itop) .NE. ' ')
	  itop	= itop + 1
	END DO
	itop	= itop - 1

	RETURN
	END


	SUBROUTINE	Epeaks_find (Ecut,ic,bigE,nbigE,
C	**********	***********
     1				     dbigE,zero_std,step_std,dbigET)

C For the efld in the /maps/ common, with given dims, we do a peaks analysis
C above the passed cut-off (Ecut).  Also use mole grid to get D.E energies.
C Put values into slot ic of the peaks arrays.
C Option to record each peak centre relative to a notional active site centre.

C The algorithm:
C Go through the kji grid, and for each point above Ecut:
C 	Skip if its already assigned to a peak, but if a 'new' one:
C		Go through cycles of peak extension, moving one grid unit
C		each time - allowed *6 nn* moves only to 'new' pts > Ecut.
C		When a cycle gives no new pts for this peak, move on.

	REAL		efld (150,150,150), mole (150,150,150),
     1			potl (150,150,150)
	REAL		peak_sums (20,5000), zero_std(3)
	INTEGER		dims (3), npeaks (20), peak_pts (20,5000),
     1			ind (150,150,150)
	INTEGER		dimsh (3), nn (6,3)
	LOGICAL		nn_do (6), solv (150,150,150)
	LOGICAL*1	itr (150,150,150)

	COMMON	/maps/		potl, efld, mole, itr, dims, step,
     1				solv
	COMMON	/peaks/		npeaks, peak_sums, peak_pts

C ind grid gives key for growth around new peak:
C	ind = 0 not yet found in peak scan (will stay 0 if <Ecut)
C	ind = 1 just found in peak scan include ir and scan around it (nn)
C	ind > 1 found and already processed

	DO 200 k		= 1, dims (3)
	  DO 200 j		= 1, dims (2)
	    DO 200 i		= 1, dims (1)
	      ind (i,j,k)	= 0
200	CONTINUE

	npeaks (ic)		= 0			! INITIALISE PK CNTR
	DO 400 k		= 1, dims (3)
	  DO 400 j		= 1, dims (2)
	    DO 400 i		= 1, dims (1)
	      IF ((efld (i,j,k).GT.Ecut).AND.(ind(i,j,k).EQ.0)) THEN
		npeaks (ic)	= npeaks (ic) + 1	! NEW PEAK
		ip		= npeaks (ic)
		peak_pts (ic,ip)   = 0
		peak_sums (ic,ip)  = 0.
		IF (npeaks (ic) .EQ. 5000) STOP 'npeaks at 5000'

		ind (i,j,k)	= 1			! MARK GROWTH PT
		DO 320 nc	= 1, 100		! GROWTH CYCLES
		  num_add	= 0			! PTS ADDED THIS CYCLE

		  DO 300 kk	= 1, dims (3)
		    dimsh (3)	= kk
		    DO 300 jj	= 1, dims (2)
		      dimsh (2)	= jj
		      DO 300 ii	= 1, dims (1)
			dimsh (1) = ii
		        IF (ind (ii,jj,kk) .EQ. 1) THEN

			  peak_pts (ic,ip) = peak_pts (ic,ip) + 1
			  en = mole(ii,jj,kk)*efld(ii,jj,kk)*
     1					      efld(ii,jj,kk)
			  peak_sums(ic,ip) = peak_sums(ic,ip)+en
			  ind (ii,jj,kk) = 2		! THIS PT IS DONE

			  CALL	nn_setup (dims, dimsh, nn, nn_do)
			  DO 280 nnn	= 1, 6		! NEIGHBOURS?
			    IF (.NOT.nn_do(nnn)) GOTO 280
			    iii		= nn (nnn,1)
			    jjj		= nn (nnn,2)
			    kkk		= nn (nnn,3)
			    IF ((efld(iii,jjj,kkk).GT.Ecut).AND.
     1				(ind(iii,jjj,kkk).EQ.0)) THEN
			      ind(iii,jjj,kkk) = 1
			      num_add	= num_add + 1
			    END IF
280			  CONTINUE
			END IF

300		  CONTINUE
		  IF (num_add .EQ. 0) GOTO 400		! PEAK DONE

320		CONTINUE
	      END IF
400	CONTINUE

C Upgrade our running sum to an energy in kJ/mole (*incomplete* for cut map).

	conv		= 1.E+14		! (mV/A)sq -> (V/m)sq
	conv		= conv * 8.854E-12	! multiply by eo
	conv		= conv * 1.E-30		! A_cubed -> m_cubed
	conv		= conv * 0.5		! half in the D.E integral
	conv		= conv * 6.022E+20	! J/molecule -> kJ/mole
	cell_vol	= step*step*step	! cell vol in A_cubed
	conv		= conv * cell_vol
	bigE		= 0.
	IF (npeaks(ic) .NE. 0) THEN
	  DO 500 ip	= 1, npeaks (ic)
	    peak_sums (ic,ip) = conv * peak_sums (ic,ip)
	    IF (peak_sums (ic,ip) .GT. bigE) THEN
	      bigE 	= peak_sums (ic,ip)
	      nbigE	= peak_pts (ic,ip)
	    END IF
500	  CONTINUE
	ELSE
	  bigE			= 0.
	  nbigE			= 0
	  dbigE			= 99.
	  dbigET		= 99.
	END IF

	RETURN
	END


	SUBROUTINE Ppeaks_find (zero_std, step_std)
C	********** ***********

C Get the +/- potl peaks and contoured patches.

	REAL		efld (150,150,150), mole (150,150,150),
     1			potl (150,150,150)
	REAL		zero_std (3)
	INTEGER		dims (3), dimsh (3), nn (6,3), ijk (3)
	LOGICAL		nn_do (6), patch_grow, solv (150,150,150),
     1			white_out
     	INTEGER		ind (150,150,150), pts (100000),
     1			patch_index(3750000,3)
	LOGICAL*1	itr (150,150,150), incl (150,150,150),
     1			swshell (150,150,150), at_writ (50000),
     2			str_flag (50000)
     	CHARACTER*60	molecule
	CHARACTER*6	atom_str (50000)
	CHARACTER*4	atyp_str (50000), res_str (50000)
	CHARACTER*1	chain_str (50000)
	REAL		xyz_str (50000,3), xyz (3), np_cut,
     1			max_potl (6), max_rad (6), abs_potl (6)
	INTEGER		nres_str (50000), cb_str (50000),sign(100000),
     1			which_patch (150,150,150)

	COMMON	/maps/		potl, efld, mole, itr, dims, step,
     1				solv
	COMMON	/ats_str/	nstr,atom_str,atyp_str,res_str,
     1				chain_str,nres_str,xyz_str,cb_str
	COMMON	/patches/	potl_cut, np_cut
	COMMON	/whitener/	white_out
	COMMON	/moley/		molecule, iml

	DATA	max_rad	/ 5., 10., 15., 20., 25., 30. /

2020	FORMAT (4I6,' = [number patches : pos : neg : np] ',
     1		2F8.1,' = [max : min (mV)]')
2040	FORMAT (/,'2DPATCH',3I6,' = [patch number : pts : sign]')
2050	FORMAT (10X,'ipt2D     x         y         z       potl')
2060	FORMAT ('2DPOINT',I6,4(F10.3))
3000	FORMAT (A6, 7X, A4, A4, A1, I4, 4X, 3F8.3, 2F6.2)
3020	FORMAT (A4, 9X, A4, A4, A1, I4, 4X, 3F8.3, 2F6.2)
3200	FORMAT (A6, 7X, A4, A4, A1, I4, 4X, 3F8.3,  F6.2, 1X,F6.1)
3300	FORMAT (' per atom list of max swshell pos potls at dist = ',
     1          '       ','   5     10     15     20     25     30',
     2           '   abs','   5     10     15     20     25     30')
3400	FORMAT (A6,7X,A4,1X,A4,1X,A1,1X,I4,4X,3F8.3,6(2X,F5.1),
     1						 4X,6(2X,F5.1))
C??3400	FORMAT (A6, 7X, A4, A4, A1, I4, 4X, 3F8.3, 6(2X,F5.1),
C??     1						4X,6(2X,F5.1))

C As for the E-field analysis, we'll process just potl pts that are NOT nn
C to the molecule grid - rather than a specific exclusion shell analysis.
C If step is around 1.5 A, then effectively like an O/N atom exclusion rad.
C The incl grid will be true if (itr) and mole>25 and {nn} mole>25.

C For the patch contouring we have to allow for diagonals within a patch,
C so later on we allow for the complete cube of nn pts - here though we
C stick at 6 (closest) nn.

	pmin			=  999999.
	pmax			= -999999.
	DO 180 k		= 1, dims (3)
	  dimsh (3)		= k
	  DO 180 j		= 1, dims (2)
	    dimsh (2)		= j
	    DO 180 i		= 1, dims (1)
	      dimsh (1)		= i
	      incl (i,j,k)	= .TRUE.
	      IF (.NOT. itr (i,j,k)) incl (i,j,k)	= .FALSE.
	      IF (.NOT.solv(i,j,k)) incl (i,j,k)	= .FALSE.
	      CALL	nn_setup (dims, dimsh, nn, nn_do)
	      DO 160 nnn	= 1, 6		! NEIGHBOURS?
		IF (.NOT.nn_do(nnn)) GOTO 160
		ii		= nn (nnn,1)
		jj		= nn (nnn,2)
		kk		= nn (nnn,3)
		IF ((.NOT.solv(ii,jj,kk))) incl (i,j,k)	= .FALSE.
160	      CONTINUE
	      IF (incl(i,j,k)) THEN
	        IF (potl(i,j,k).GT.pmax) pmax = potl(i,j,k)
	        IF (potl(i,j,k).LT.pmin) pmin = potl(i,j,k)
	      END IF
180	CONTINUE

C Put in some repeat-ish stuff here to get an almost shrink-wrapped shell
C for our statistical analysis - important that we try to get the distn of
C pts that could be generating potl/efld pks - take mol, skip nn, then shell.
C Much of this analysis we just copy from E peaks where we didnt use sw.

	DO 480 k		= 1, dims (3)
	  DO 480 j		= 1, dims (2)
	    DO 480 i		= 1, dims (1)
	      swshell (i,j,k)	= .FALSE.
480	CONTINUE						! End i,j,k

	DO 500 k		= 1, dims (3)
	  dimsh (3)		= k
	  DO 500 j		= 1, dims (2)
	    dimsh (2)		= j
	    DO 500 i		= 1, dims (1)
	      dimsh (1)		= i
	      IF ((itr(i,j,k)) .AND. (solv(i,j,k)) .AND.
     1		  (.NOT.incl(i,j,k))) THEN	! on nn1 to molecule shell
		CALL	nn_setup (dims, dimsh, nn, nn_do)
		DO 460 nnn	= 1, 6		! NEIGHBOURS?
		  IF (.NOT.nn_do(nnn)) GOTO 460
		  ii		= nn (nnn,1)
		  jj		= nn (nnn,2)
		  kk		= nn (nnn,3)

Coct03 - small change - could have had nn2+nn1 shells - want nn2 only
Coct03		  IF ((itr(ii,jj,kk)).AND.(solv(ii,jj,kk)))
Coct03     1		    swshell (ii,jj,kk) = .TRUE.
		  IF ((itr(ii,jj,kk)).AND.(solv(ii,jj,kk)).AND.
     1		      (incl(ii,jj,kk))) swshell (ii,jj,kk) = .TRUE.

460		CONTINUE
	      END IF
500	CONTINUE						! End i,j,k

C?? wednesday

C ind grid gives key for growth around new patch:
C	ind = 0 not yet found in peak scan (will stay 0 if ABS<potl_cut
C	and ABS>np_cut
C	ind = 1 just found in peak scan include it and scan around it (nn)
C	ind > 1 found and already processed
C which_patch records the patch number for a pt (0 means no patch at
C 	these thresholds: potl_cut and np_cut)

C We've decided just to do 2D contours for now - that is what we want for
C patches - if we need 3D contours later, then just adapt the code to not
C process for being on the nn2 shell grid (swshell=T).

	DO 520 k		= 1, dims (3)
	  DO 520 j		= 1, dims (2)
	    DO 520 i		= 1, dims (1)
	      ind (i,j,k)	= 0
	      which_patch (i,j,k) = 0
520	CONTINUE

	npatchP			= 0			! Positive
	npatchN			= 0			! Negative
	npatchH			= 0			! Hydrophobic
	np			= 0
	ntot			= 0
	DO 600 k		= 1, dims (3)		! init patch centre
	  DO 600 j		= 1, dims (2)
	    DO 600 i		= 1, dims (1)
	      patch_grow	= .FALSE.
C could do 3D by taking out the swshell in following code
	      IF (incl(i,j,k) .AND. (ind(i,j,k).EQ.0) .AND.
     1		  swshell(i,j,k)) THEN    	! solv & unset & nn2
	        IF      (potl (i,j,k) .GT.  potl_cut) THEN
	       	  npatchP	= npatchP + 1
		  np		= np + 1
		  sign (np)	=  1
		  patch_grow	= .TRUE.
	        ELSE IF (potl (i,j,k) .LT. -potl_cut) THEN
	          npatchN	= npatchN + 1
		  np		= np + 1
		  sign (np)	= -1
	          patch_grow	= .TRUE.
		ELSE
		  IF (ABS(potl(i,j,k)) .LT. np_cut) THEN
	            npatchH	= npatchH + 1
		    np		= np + 1
		    sign (np)	= 0
	            patch_grow	= .TRUE.
		  END IF
	        END IF
		IF (patch_grow) THEN    		! 2D patch_grow
		  IF (np .GT. 100000) STOP 'more than 100000 patches'
		  pts (np)   	= 0
		  ind (i,j,k)	= 1			! mark growth point
		  DO 560 ncyc	= 1, 100		! growth cycles
		    num_add	= 0			! pts added this cycle
		    DO 540 kk		= 1, dims (3)
		      dimsh (3)		= kk
		      DO 540 jj		= 1, dims (2)
		        dimsh (2)	= jj
		        DO 540 ii	= 1, dims (1)
			  dimsh (1) = ii
			  IF (ind (ii,jj,kk) .EQ. 1) THEN
			    pts (np) = pts (np) + 1
			    ind (ii,jj,kk) = 2		! this pt is done
			    which_patch (ii,jj,kk) = np
			    ntot	= ntot + 1
			    patch_index (ntot,1)	= ii
			    patch_index (ntot,2)	= jj
			    patch_index (ntot,3)	= kk

Cnn			    CALL nn_setup (dims, dimsh, nn, nn_do)
Cnn			    DO 530 nnn	= 1, 6		! NEIGHBOURS?
Cnn			      IF (.NOT.nn_do(nnn)) GOTO 530
Cnn			      iii	= nn (nnn,1)
Cnn			      jjj	= nn (nnn,2)
Cnn			      kkk	= nn (nnn,3)
C if we don't do the cube (below), the swshell patches will break up -
C assuming that we have it right anyway - needs checking

			    DO 530 kkk	= kk-1, kk+1
			    DO 530 jjj	= jj-1, jj+1
			    DO 530 iii	= ii-1, ii+1

			      IF (incl(iii,jjj,kkk).AND.
     1			          (ind(iii,jjj,kkk).EQ.0).AND.
     2			       swshell(iii,jjj,kkk)) THEN
			        IF      (sign(np) .EQ. 1) THEN
				  IF (potl(iii,jjj,kkk).GT.potl_cut)
     1				     THEN
				    ind(iii,jjj,kkk) = 1
				    num_add	= num_add + 1
				  END IF
			        ELSE IF (sign(np) .EQ.-1) THEN
				 IF (potl(iii,jjj,kkk).LT.-potl_cut)
     1				     THEN
				    ind(iii,jjj,kkk) = 1
				    num_add	= num_add + 1
				  END IF
				ELSE IF (sign(np) .EQ. 0) THEN
				 IF (ABS(potl(iii,jjj,kkk)).LT.np_cut)
     1				     THEN
				    ind(iii,jjj,kkk) = 1
				    num_add	= num_add + 1
				  END IF
				ELSE
				  STOP 'sign not set'
				END IF
			      END IF			! end incl/ind=0
530			    CONTINUE			! end nn
			  END IF			! if ind eq 1
540		    CONTINUE				! end ii,jj,kk
		    IF (num_add .EQ. 0) GOTO 600	! PEAK DONE
560		  CONTINUE				! end growth cycles
		END IF					! end 2D patch_grow
	      END IF					! end solv & unset
600	CONTINUE					! end i,j,k

C Write out all swshell pts to pdb file.

	OPEN (UNIT=8, NAME=molecule(1:iml)//'_SHELL.pdb', TYPE='NEW')
	IF (white_out) THEN
	  WRITE (8,*) ' SHELL colours: 10-plus 25-np 40-minus'
	ELSE
	  WRITE (8,*) ' SHELL colours: 10-plus 40-minus 25-np 33-other'
	END IF

	nres_pts		= 0
	DO 610 k		= 1, dims (3)		! init patch centre
	  ijk (3)		= k
	  DO 610 j		= 1, dims (2)
	    ijk (2)		= j
	    DO 610 i		= 1, dims (1)
	      ijk (1)		= i
	      IF (swshell(i,j,k)) THEN
	        DO 606 n	= 1, 3
	          xyz (n) = zero_std(n) + step_std*FLOAT((ijk(n)-1))
606	        CONTINUE				! End n
		IF      (potl(i,j,k).GT. potl_cut) THEN
		  col_here	= 10.
		ELSE IF (potl(i,j,k).LT.-potl_cut) THEN
		  col_here	= 40.
		ELSE IF (ABS(potl(i,j,k)).LT.np_cut) THEN
		  col_here	= 25.
		ELSE
		  IF (white_out) THEN
		    col_here	= 25.
		  ELSE
		    col_here	= 33.
		  END IF
		END IF
		nres_pts	= nres_pts + 1
		WRITE (8,3020) 'ATOM','CA  ','CA  ','A', nres_pts,
     1			   (xyz(n),n=1,3), occ, col_here
	      END IF
610	CONTINUE					! End i,j,k
	CLOSE (UNIT=8)

C Write out the patch overall information, and then the patches, pt by pt!

	OPEN (UNIT=8, NAME=molecule(1:iml)//'_PATCHES.pdb', TYPE='NEW')
	IF (white_out) THEN
	  WRITE (8,*) ' PATCH colours: 10-plus 25-np 40-minus'
	ELSE
	  WRITE (8,*) ' PATCH colours: 10-plus 40-minus 25-np 33-other'
	END IF

	itot		= 0
	nres_pts	= 0
	occ		= 1.0
	WRITE (1,2020) np, npatchP, npatchN, npatchH, pmax, pmin
	DO 640 ip	= 1, np
	  WRITE (1,2040) ip, pts(ip), sign(ip)
	  WRITE (1,2050)
	  DO 630 ipt	= 1, pts(ip)
	    itot	= itot + 1
	    DO 620 n	= 1, 3
	      ijk (n)	= patch_index (itot,n)
	      xyz (n)	= zero_std(n) + step_std*FLOAT((ijk(n)-1))
620	    CONTINUE					! End n
	    IF (which_patch(ijk(1),ijk(2),ijk(3)) .NE. ip)
     1	        STOP 'which patch mismatch'
	    IF (.NOT.swshell(ijk(1),ijk(2),ijk(3))) STOP 'screwup'
	    WRITE (1,2060) ipt,(xyz(n),n=1,3),
     1		           potl(ijk(1),ijk(2),ijk(3))

	    IF      (potl(ijk(1),ijk(2),ijk(3)).GT. potl_cut) THEN
	      col_here	= 10.
	    ELSE IF (potl(ijk(1),ijk(2),ijk(3)).LT.-potl_cut) THEN
	      col_here	= 40.
	    ELSE IF (ABS(potl(ijk(1),ijk(2),ijk(3))).LT.np_cut) THEN
	      col_here	= 25.
	    ELSE
	      IF (white_out) THEN
	        col_here	= 25.
	      ELSE
	        col_here	= 33.
	      END IF
	    END IF
	    nres_pts	= nres_pts + 1
	    WRITE (8,3020) 'ATOM','CA  ','CA  ','A', nres_pts,
     1			   (xyz(n),n=1,3), occ, col_here

630	  CONTINUE					! End ipt
640	CONTINUE					! End ip
	CLOSE (UNIT=8)

C Make a repeat coordinate file (same space as original) colour-coded by
C patch type - to check what's going on graphically.
C ?? Currently, we're doing a VERY INEFFICIENT whole xyz vs ijk search.

	occ			= 1.0
	IF (white_out) THEN
	  WRITE (4,*) ' PATCH colours: 10-plus 25-np 40-minus'
	ELSE
	  WRITE (4,*) ' PATCH colours: 10-plus 40-minus 25-np 33-other'
	END IF
	WRITE (7,3300)
	DO 700 iat		= 1, nstr
	  dist_min		= 9999.
	  col_here		= 99.
	  DO 650 imax		= 1, 6
	    max_potl (imax)	= 0.
	    abs_potl (imax)	= 0.
650	  CONTINUE					! End imax
	  DO 660 k		= 1, dims (3)
	    zg			= zero_std(3) + step_std*FLOAT(k)
	    DO 660 j		= 1, dims (2)
	      yg		= zero_std(2) + step_std*FLOAT(j)
	      DO 660 i		= 1, dims (1)
		xg		= zero_std(1) + step_std*FLOAT(i)
		IF (swshell(i,j,k)) THEN
		  dist		= SQRT( (xyz_str(iat,1)-xg)**2 +
     1					(xyz_str(iat,2)-yg)**2 +
     2					(xyz_str(iat,3)-zg)**2 )
		  IF (dist .LT. dist_min) THEN
		    dist_min	= dist
		    IF      (potl(i,j,k).GT. potl_cut) THEN
		      col_here	= 10.
		    ELSE IF (potl(i,j,k).LT.-potl_cut) THEN
		      col_here	=40.
		    ELSE IF (ABS(potl(i,j,k)).LT.np_cut) THEN
		      col_here	= 25.
		    ELSE
		      IF (white_out) THEN
		        col_here	= 25.
		      ELSE
		        col_here	= 33.
		      END IF
		    END IF
		  END IF
		  DO 656 imax	= 1, 6
		    IF (dist .LT. max_rad(imax)) THEN
		      IF (potl(i,j,k) .GT. max_potl(imax)) THEN
		        max_potl(imax)	= potl(i,j,k)
		      END IF
		      IF (ABS(potl(i,j,k)).GT.abs_potl(imax)) THEN
		        abs_potl(imax)	= ABS (potl(i,j,k))
		      END IF
		    END IF
656		  CONTINUE					! End imax
	        END IF
660	  CONTINUE						! End ijk
	  IF (col_here .EQ. 99.) STOP 'colour code not set'
C??	  IF (atom_str(iat) .EQ. 'ATOM  ') THEN
	  IF ((atom_str(iat) .EQ. 'ATOM  ') .OR.
     1	      (atom_str(iat) .EQ. 'HETATM')) THEN
	    nres_owt	= nres_str(iat)
C Oct 2007 - reduce I4 res nos to I4 (e.g. for moved to end Z chain res)
C	    IF (nres_str(iat) .GE. 2000) THEN
C	      nres_owt	= nres_str(iat) - 2000
C	    ELSE IF (nres_str(iat) .GE. 1000) THEN
C	      nres_owt	= nres_str(iat) - 1000
C	    ELSE
C	      nres_owt	= nres_str(iat)
C	    END IF
	    WRITE (4,3000) atom_str(iat), atyp_str(iat), res_str(iat),
     1			   chain_str(iat), nres_str(iat),
     2			   (xyz_str(iat,n),n=1,3), occ, col_here
	    WRITE (7,3400) atom_str(iat), atyp_str(iat), res_str(iat),
     1			   chain_str(iat), nres_owt,
     2			   (xyz_str(iat,n),n=1,3),
     3			   (max_potl(imax),imax=1,6),
     4			   (abs_potl(imax),imax=1,6)
	  END IF
700	CONTINUE						! End i
	CLOSE (UNIT=4)
	CLOSE (UNIT=7)

	RETURN
	END


	SUBROUTINE	nn_setup (dims, dimsh, nn, nn_do)
C	**********	********

C Just set up indices for 6 nn points according to whether at edge or not.

	INTEGER		dims (3), dimsh (3), nn (6,3)
	LOGICAL		nn_do (6)

	DO 200 nnn	= 1, 6
	  nn (nnn,1)	= dimsh (1)
	  nn (nnn,2)	= dimsh (2)
	  nn (nnn,3)	= dimsh (3)
	  nn_do (nnn)	= .TRUE.
200	CONTINUE

	IF (dimsh (1) .EQ. 1) THEN
	  nn_do (1)	= .FALSE.
	ELSE
	  nn (1,1)	= dimsh (1) - 1
	END IF
	IF (dimsh (1) .EQ. dims (1)) THEN
	  nn_do (2)	= .FALSE.
	ELSE
	  nn (2,1)	= dimsh (1) + 1
	END IF

	IF (dimsh (2) .EQ. 1) THEN
	  nn_do (3)	= .FALSE.
	ELSE
	  nn (3,2)	= dimsh (2) - 1
	END IF
	IF (dimsh (2) .EQ. dims (2)) THEN
	  nn_do (4)	= .FALSE.
	ELSE
	  nn (4,2)	= dimsh (2) + 1
	END IF

	IF (dimsh (3) .EQ. 1) THEN
	  nn_do (5)	= .FALSE.
	ELSE
	  nn (5,3)	= dimsh (3) - 1
	END IF
	IF (dimsh (3) .EQ. dims (3)) THEN
	  nn_do (6)	= .FALSE.
	ELSE
	  nn (6,3)	= dimsh (3) + 1
	END IF

	RETURN
	END

